#  Copyright (c) 2024. Aron Barocsi | All rights reserved.

from xposer.api.base.base_kafka_service import BaseKafkaService


class SampleAppKafkaService(BaseKafkaService):
    """Some example so developer may override"""
    ...
